<?php
namespace Teknisa\FIS\Modules\Acessorias\Services;

use Teknisa\FIS\Modules\Acessorias\Factories\Fis93000DirfFactory;
use Teknisa\FIS\Modules\Acessorias\Services\Fis91000ObrigacoesAcessoriasService;
use Teknisa\FIS\Modules\Acessorias\Services\Agendamento;

use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Libs\Util\NewCode;
use \Teknisa\Libs\Util\Libraries\PHPMailer;

class Fis93000DirfService {

    protected $environment;
    protected $dirfFactory;
    protected $obrigacoesService;

    public $mensagemBody = '';
    public $mensagemBodySucess = '';
    public $mensagemError = '';
    public $mensagemErrorBody = '';
    public $geraArquivo = '';

    public function __construct(Environment $environment, Fis93000DirfFactory $dirfFactory, Fis91000ObrigacoesAcessoriasService $obrigacoesService, NewCode $newCode, Agendamento $agendamento ) 
    {
        $this->environment              = $environment;
        $this->dirfFactory             = $dirfFactory;
        $this->obrigacoesService        = $obrigacoesService;
        $this->newCode                  = $newCode;
        $this->agendamento              = $agendamento;
    }

    public function savedirf($row) {
        try {
            
            $row['CDTELA'] = 'FIS93000';
            
            $dtinicial  = DateUtil::getPrimeiroDiaDoMes( $this->formataData($row['DTREFINI']));
            $dtfinal    = DateUtil::getUltimoDiaDoMes( $this->formataData($row['DTREFINI']));
            $dtinicial = $dtinicial->format("d/m/Y");
            $dtfinal = $dtfinal->format("d/m/Y");
    
            $row["DTINICIAL"] = $dtinicial;
            $row["DTFINAL"] = $dtfinal;
            
            //var_dump($row);
            //throw new \Exception('ioioioioioi');
            
            if(isset($row['__is_new']) && $row['__is_new'] == true){
                $this->dirfFactory->insert($row);
            } else {
                $this->dirfFactory->update($row);
        
            }
        
            
        } catch(\Exception $e) {
            //
            throw $e;
        }
    }
    
    
    public function exportardirf($row) {
        // Valida os dados enviados na requisição.
        $this->validatedirf($row);

        // Gera o próximo código sequencial do arquivo magnético.
        $row["NRSEQARQMAG"] = $this->obrigacoesService->getNextNrseqarqmag($row["NRORG"]);

        // Monta o array com os parâmetros para chamar a procedure.
        $params = array(
            "NRORG"         => $row["NRORG"],
            "CDTELA"        => $row["CDTELA"],
            "CDEMPRESA"     => $row["CDEMPRESA"],
            "CDINSCRICAO"    => $row["CDINSCRICAO"],
            
            //Campos da tabela que serão salvos e os campos da declaração
            "DTINICIAL" => $row["DTINICIAL"],
            "DTFINAL"   => $row["DTFINAL"],
            
            "CDCAMPO1"  => $row["CDTPDECLARACAO"],
            "CDCAMPO2"  => $row["NUMRECIBO"],
            "CDCAMPO3"  => $row["CDNATDECLARANTE"],
            "CDCAMPO4"  => $row["CDINDSOCIO"],
            "CDCAMPO5"  => $row["CDDEPOSITCRED"],
            "CDCAMPO6"  => $row["CDTPINVEST"],
            "CDCAMPO7"  => $row["CDRENDPGEXT"],
            "CDCAMPO8"  => $row["CDINDPLANSD"],
            "CDCAMPO9"  => $row["CDPGIMUISE"],
            "CDCAMPO10"  => $row["CDFUNPUBDIRPRIV"],
            "CDCAMPO11"  => $row["CDFUNPUBCDSITESPDIRPRIV"],
            "CDCAMPO12"  => $row["CDREPRES"],
            "DTCAMPO1"=>$row["DTEVENTO"]
        );

        // Chama a procedure de exportação.
        $this->dirfFactory->callExportacaoProcedure($params);

        // Foi evitado utilizar um bloco finally devido a possíveis
        // incompatibilidades com a versão do PHP.
        try{

            // Chama a função para gerar o arquivo texto.
            $fileName = $this->createFile($row);

            // Retorna o nome do arquivo com suas possíveis modificações.
            return $fileName;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    private function validatedirf($row) {
        if (isset($row['NRAGENDAMENTO'])){
            
            $this->mensagemError = '';
            
            // Verifica se o campo "Empresa" possui algum valor informado.
            if (empty($row["CDEMPRESA"])) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Empresa deve ser informada.';
                $this->geraArquivo = false;
                //throw new \Exception("Empresa deve ser informada.");
            }
            // Verifica se o campo "Inscrição" possui algum valor informado.
            if (empty($row["CDINSCRICAO"])) {
                $this->mensagemError = $this->mensagemError.'<br />'.'CDINSCRICAO deve ser informada.';
                $this->geraArquivo = false;
                //throw new \Exception("Inscrição deve ser informada.");
            }
    
            if (empty($row["DTREFINI"])) {
                $this->mensagemError = $this->mensagemError.'<br />'.'Referência deve ser informada.';
                $this->geraArquivo = false;
                //throw new \Exception("Referência deve ser informada.");
            }
            
            // Verifica se o campo "Nome do Arquivo" possui algum valor informado.
            if (empty($row["NMARQUIVO"])) {
                throw new \Exception("Nome do Arquivo deve ser informado.");
            }
    
    
            
            $datas = $this->obrigacoesService->getFirstAndLastDays($row["DTREFINI"]);
            $row["DTINICIAL"] = $datas["DTINICIAL"];
            $row["DTFINAL"] = $datas["DTFINAL"];
        
        } else {
            // Verifica se o campo "Empresa" possui algum valor informado.
            if (empty($row["CDEMPRESA"])) {
                throw new \Exception("Empresa deve ser informada.");
            }
           
            // Verifica se o campo "Referência" possui algum valor informado.
            if (empty($row["DTREFINI"])) {
                throw new \Exception("Referência deve ser informada.");
            }
    
            // Verifica se o campo "Nome do Arquivo" possui algum valor informado.
            if (empty($row["NMARQUIVO"])) {
                throw new \Exception("Nome do Arquivo deve ser informado.");
            }
    
            // Busca e atribui os valores do NRORG e datas inicial e final do mês
            // à variável row.
            $nrorg = $this->environment->getNrorg();
            $datas = $this->obrigacoesService->getFirstAndLastDays($row["DTREFINI"]);
            $row["DTINICIAL"] = $datas["DTINICIAL"];
            $row["DTFINAL"] = $datas["DTFINAL"];
            $row["NRORG"] = $nrorg;
    
        }
    }
    
    /*
     * Função para criar o arquivo da exportação da GIA.
     */
    private function createFile($row) {
        // Cria os parâmetros para buscar os dados gerados na procedure.
        $params = array(
            "NRSEQARQMAG" => $row["NRSEQARQMAG"],
            "NRORG" => $row["NRORG"]
        );
        $dirfData = $this->dirfFactory->getdirf($params);
        
        if(isset($row['NRAGENDAMENTO']) && empty($dirfData)){
            $this->mensagemError = '<br />'."Não há dados a serem exibidos.";
            return $fileName = false;
        }

        // Chama a função para a criação do arquivo.
        $fileName = $this->obrigacoesService->createFileExportacao($row["NMARQUIVO"], $dirfData, "DSREGARQMAG");

        // Retorna o nome do arquivo exportado.
        return $fileName;
    }
    

    
}